import time, io

import ipywidgets.widgets as widgets

from escape.utils.widgets.base import EscapeObjectLayout
from escape.utils.widgets.modelstack import MSObjectLayout
from escape.utils.widgets.plot import Plot2dWidget
from ipywidgets.widgets.widget_box import VBox


class RegressorObjectLayout(MSObjectLayout):
    
    def __init__(self, escobj, **kwargs):
        modelstack = escobj.modelstack
        self.regressor = escobj
        self.kwargs=kwargs
        self.modelstack = modelstack
        MSObjectLayout.__init__(self, modelstack, **kwargs)
        self.update()
    
    def init_header(self, **kwargs):

        def on_train_click(btn):
            try:
                self.trainbtn.disabled = True
                self.resetbtn.disabled = True
                self.shakebtn.disabled = True
                self.nextbtn.disabled = True
                self.prevbtn.disabled = True
                self.disable_parameters(True)
                
                score=self.regressor.train(True)
                self.status_label.value = '<b>Score:</b> {0:.4g}'.format(score)
                self.update_parameter_sliders(silent=True)
                self._update(buf=False)
            except Exception as err:
                print(err)
            finally:
                self.trainbtn.disabled = False
                self.resetbtn.disabled = False
                self.shakebtn.disabled = False
                self.nextbtn.disabled = False
                self.prevbtn.disabled = False
                self.disable_parameters(False)
                
        def on_next_click(btn):
            try:
                self.regressor.next_test()
                self.update_parameter_sliders(silent=True)
                self._update(buf=False)
            except Exception as err:
                print(err)
                
        def on_prev_click(btn):
            try:
                self.regressor.prev_test()
                self.update_parameter_sliders(silent=True)
                self._update(buf=False)
            except Exception as err:
                print(err)
            
        def on_reset_click(btn):
            try:
                self.regressor.reset()
                self.update_parameter_sliders(silent=True)
                self._update(buf=False)
            except Exception as err:
                print(err)
            
        def on_shake_click(btn):
            try:
                self.modelstack.shake()
                self.update_parameter_sliders(silent=True)
                self._update(buf=False)
            except Exception as err:
                print(err)
        
        btnlay = widgets.Layout( width='40px' )
            
        self.trainbtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Train regressor and predict',
                icon='train',
                layout=btnlay)
        self.trainbtn.on_click(on_train_click)
        
        self.nextbtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Shows solution for the next test data',
                icon='arrow-right',
                layout=btnlay)
        self.nextbtn.on_click(on_next_click)
        
        self.prevbtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Shows solution for the prev test data',
                icon='arrow-left',
                layout=btnlay)
        self.prevbtn.on_click(on_prev_click)
        
        self.resetbtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Reset parameters values',
                icon='refresh',
                layout=btnlay)
        
        self.resetbtn.on_click(on_reset_click)
        
        self.shakebtn = widgets.Button(
                description='',
                disabled=False,
                tooltip='Shake parameters values',
                icon='random',
                layout=btnlay)
        
        self.shakebtn.on_click(on_shake_click)
                
        toolbar = widgets.HBox(children=[
            self.trainbtn,
            self.prevbtn,
            self.nextbtn,
            self.resetbtn,
            self.shakebtn,
            ])
        
        return toolbar
    
    def init_center(self, **kwargs):
        plots = super(RegressorObjectLayout, self).init_center(**kwargs)
        return plots
    
    def init_footer(self, **kwargs):
        status_label = widgets.HTML('<b>Score: </b>')
        self.status_label = status_label
        return VBox([status_label])
    